unit FadeImage;

interface

uses
  ExtCtrls, Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

Type

  TRGB=record
  b,g,r : byte;
end;
 type
    ARGB=array [0..1] of TRGB;
    PARGB=^ARGB;

type
  TFadeImage = class(TGraphicControl)
  private
    P1,P2,P3 : Pargb;
    FPicture: TBitmap;
    FSPicture : TBitmap;
    FNewPicture : TBitmap;
    FTimer : TTimer;
    FSteps: Integer;
    FInterval: Integer;
    FNewStep : Integer;
    FColor: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnMouseEnter: TNotifyEvent;
    FOnEndAnimate: TNotifyEvent;
    procedure SetPicture(const Value: TBitmap);
    procedure SetSteps(const Value: Integer);
    procedure SetInterval(const Value: Integer);
    Procedure NewSlide(Sender : TObject);
    procedure SetColor(const Value: TColor);
    procedure setonmouseleave(const Value: TNotifyEvent);
    procedure setOnMouseEnter(const Value: TNotifyEvent);
    procedure SetOnEndAnimate(const Value: TNotifyEvent);
    { Private declarations }
  protected
  procedure paint; override;
    { Protected declarations }
  public
  Constructor Create(AOwner : TComponent); override;
  procedure CMMOUSELEAVE( var Message: TWMNoParams); message CM_MOUSELEAVE;
  procedure CMMOUSEEnter(var Message: TWMNoParams); message CM_MOUSEenter;
    { Public declarations }
  published
  Property Picture : TBitmap read FPicture Write SetPicture;
  Property Steps : Integer Read FSteps Write SetSteps;
  Property Interval : Integer Read FInterval Write SetInterval;
  Property DefaultColor : TColor Read FColor Write SetColor;
  Property Hint;
  Property ShowHint;
  Property Enabled;
  Property OnMouseLeave : TNotifyEvent read FOnMouseLeave write setonmouseleave;
  Property OnMouseEnter : TNotifyEvent read FOnMouseEnter write setOnMouseEnter;
  Property OnClick;
  Property OnMouseMove;
  Property OnMouseDown;
  Property OnMouseUp;
  Property PopupMenu;
  property visible;
  Property OnEndAnimate : TNotifyEvent read FOnEndAnimate Write SetOnEndAnimate;
  procedure setpicturewithoutfade(image : Tbitmap);
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Dm', [TFadeImage]);
end;

{ TFadeImage }

procedure TFadeImage.CMMOUSEEnter(var Message: TWMNoParams);
begin
 If Assigned(FOnMouseEnter) then FOnMouseEnter(self);
end;

procedure TFadeImage.CMMOUSELEAVE(var Message: TWMNoParams);
begin
 If Assigned(FOnMouseLeave) then FOnMouseLeave(self);
end;

constructor TFadeImage.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle + [csReplicatable];
  Width:=100;
  Height:=100;
  FSteps:=50;
  FTimer:=TTimer.Create(self);
  FTimer.enabled:=false;
  FTimer.Interval:=55;
  FTimer.OnTimer := NewSlide;
  FPicture:=TBitmap.create;
  FPicture.pixelformat:=pf24bit;
  FSPicture:=TBitmap.create;
  FSPicture.pixelformat:=pf24bit;
  FNewPicture:=TBitmap.create;
  FNewPicture.pixelformat:=pf24bit;
  FNewStep:=0;
end;

procedure TFadeImage.NewSlide(Sender: TObject);
var i,j : integer;
begin
  Inc(FNewStep);
  for i:=0 to FSpicture.height-1 do
 begin
 p2:=FNewPicture.scanline[i];
 p1:=FPicture.scanline[i];
 p3:=FSpicture.scanline[i];
 for j:=0 to FSpicture.width-1 do
  begin
    p3[j].r:=p1[j].r-round((p1[j].r-p2[j].r)*FNewStep/FSteps);
    p3[j].g:=p1[j].g-round((p1[j].g-p2[j].g)*FNewStep/FSteps);
    p3[j].b:=p1[j].b-round((p1[j].b-p2[j].b)*FNewStep/FSteps);
  end;
 end;
 With canvas do
 draw(0,0,FSpicture);
  If FNewStep>=FSteps then
  begin
   FTimer.Enabled:=false;
   FNewStep:=0;
   FPicture.assign(FNewPicture);
   If assigned(OnEndAnimate) then OnEndAnimate(self);
  end;
end;

procedure TFadeImage.paint;
begin
  With inherited Canvas do
  If not Ftimer.enabled then
  begin
  If Fpicture<>nil then draw(0,0,Fpicture)
  end else begin
  If FSpicture<>nil then draw(0,0,FSpicture)
  end;
end;

procedure TFadeImage.SetColor(const Value: TColor);
begin
  FColor := Value;
end;

procedure TFadeImage.SetInterval(const Value: Integer);
begin
  If value>0 then
  FInterval := Value else FInterval:=1;
  FTimer.Interval:=FInterval;
end;

procedure TFadeImage.SetOnEndAnimate(const Value: TNotifyEvent);
begin
  FOnEndAnimate := Value;
end;

procedure TFadeImage.setOnMouseEnter(const Value: TNotifyEvent);
begin
  FOnMouseEnter := Value;
end;

procedure TFadeImage.setonmouseleave(const Value: TNotifyEvent);
begin
  FOnMouseLeave := Value;
end;

procedure TFadeImage.SetPicture(const Value: TBitmap);
begin
  If FPicture=nil then
  FPicture.assign(Value) else
  begin
   FNewStep:=0;
   FSPicture.Canvas.Brush.color:=fcolor;
   FPicture.Canvas.Brush.color:=fcolor;
   FPicture.assign(FSPicture);
   FNewPicture.assign(Value);
   FTimer.enabled:=true;
   FSPicture.width:=FNewPicture.width;
   FSPicture.Height:=FNewPicture.Height;
   FPicture.width:=FNewPicture.width;
   FPicture.Height:=FNewPicture.Height;
  end;
end;

procedure TFadeImage.setpicturewithoutfade(image: Tbitmap);
begin
 if image=nil then exit;
  (inherited Canvas).draw(0,0,image);
  FSPicture.width:=image.width;
  FSPicture.Height:=image.Height;
  FPicture.width:=image.width;
  FPicture.Height:=image.Height;
  FPicture.assign(image);
  If assigned(OnEndAnimate) then OnEndAnimate(self);
end;

procedure TFadeImage.SetSteps(const Value: Integer);
begin
  If Value>2 then
  FSteps := Value else FSteps:=50;
end;

end.
